import networkx as nx

rj = nx.Graph()
rj.add_edge("Juliet", "Nurse")
rj.add_edge("Juliet", "Tybalt")
rj.add_edge("Juliet", "Capulet")
rj.add_edge("Juliet", "Friar Laurence")
rj.add_edge("Juliet", "Romeo")
rj.add_edge("Capulet", "Tybalt")
rj.add_edge("Capulet", "Paris")
rj.add_edge("Capulet", "Escalus")
rj.add_edge("Romeo", "Friar Laurence")
rj.add_edge("Romeo", "Benvolio")
rj.add_edge("Romeo", "Montague")
rj.add_edge("Romeo", "Mercutio")
rj.add_edge("Montague", "Benvolio")
rj.add_edge("Montague", "Escalus")
rj.add_edge("Mercutio", "Escalus")
rj.add_edge("Mercutio", "Paris")
rj.add_edge("Escalus", "Paris")


def friends(graph, user):
    """Returns a set of the friends of the given user, in the given graph."""
    return set(graph.neighbors(user))

def friends_of_friends(graph, user):
    """Returns a set of friends of friends of the given user, in the given graph. The result does not include the user nor their friends """
    fof = set()
    f = friends(graph, user)
    for fren in f:
        friend = friends(graph, fren)
        fof = fof | friend
    f.add(set([user]))
    g = (fof - f)
    return g

friends_of_friends(rj, "Mercutio")

#Traceback (most recent call last):
#  File "unhashable_type.py", line 38, in <module>
#    friends_of_friends(rj, "Mercutio")
#  File "unhashable_type.py", line 34, in friends_of_friends
#    f.add([user])
#TypeError: unhashable type: 'set'

# This is an easy one:
# Whenever you see an "unhashable type" error,
# it means you are using a mutable data type
# where only an immutable data type is allowed.
# Usually, a dictionary or a set.
# Also, the line number where the error is
# caught is usually all you need to look at.

# Debugging approach:
# Check the type of the argument to add.  Work backwards to
# figure out where this variable receives its current value.
